@@ -7,6 +7,7 @@
 #include "to_string.hpp"
 #include "inspect.hpp"
 #include "eval.hpp"
+#include "utf8_string.hpp"
 
 #include <cmath>
 #include <cctype>
@@ -36,7 +37,7 @@ namespace Sass {
                                     false);
   }
 
-  Definition* make_c_function(Signature sig, Sass_C_Function f, Context& ctx)
+  Definition* make_c_function(Signature sig, Sass_C_Function f, void* cookie, Context& ctx)
   {
     Parser sig_parser = Parser::from_c_str(sig, ctx, "[c function]");
     sig_parser.lex<Prelexer::identifier>();
@@ -48,6 +49,7 @@ namespace Sass {
                                     name,
                                     params,
                                     f,
+                                    cookie,
                                     false, true);
   }
 
@@ -688,6 +690,142 @@ namespace Sass {
       return result;
     }
 
+
+    Signature str_length_sig = "str-length($string)";
+    BUILT_IN(str_length)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      string str = s->value();
+      size_t length_of_s = str.size();
+      size_t i = 0;
+
+      if (s->is_quoted()) {
+        ++i;
+        --length_of_s;
+      }
+
+      size_t len = UTF_8::code_point_count(str, i, length_of_s);
+
+      return new (ctx.mem) Number(path, position, len);
+    }
+
+    Signature str_insert_sig = "str-insert($string, $insert, $index)";
+    BUILT_IN(str_insert)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      string str = s->value();
+      char quotemark = s->quote_mark();
+      str = unquote(str);
+      String_Constant* i = ARG("$insert", String_Constant);
+      string ins = i->value();
+      ins = unquote(ins);
+      Number* ind = ARG("$index", Number);
+      double index = ind->value();
+      size_t len = UTF_8::code_point_count(str, 0, str.size());
+
+      if (index > 0 && index <= len) {
+        // positive and within string length
+        str.insert(UTF_8::code_point_offset_to_byte_offset(str, index-1), ins);
+      } 
+      else if (index > len) {
+        // positive and past string length
+        str += ins;
+      }
+      else if (index == 0) {
+        str = ins + str;
+      }
+      else if (std::abs(index) <= len) {
+        // negative and within string length
+        index += len + 1;
+        str.insert(UTF_8::code_point_offset_to_byte_offset(str, index), ins);
+      }
+      else {
+        // negative and past string length
+        str = ins + str;
+      }
+
+      if (quotemark) {
+        str = quote(str, quotemark);
+      }
+
+      return new (ctx.mem) String_Constant(path, position, str);
+
+    }
+
+    Signature str_index_sig = "str-index($string, $substring)";
+    BUILT_IN(str_index)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      String_Constant* t = ARG("$substring", String_Constant);
+      string str = s->value();
+      str = unquote(str);
+      string substr = t->value();
+      substr = unquote(substr);
+
+      size_t c_index = str.find(substr);
+      if(c_index == string::npos) {
+        return new (ctx.mem) Null(path, position);
+      }
+      size_t index = UTF_8::code_point_count(str, 0, c_index + 1);
+
+      return new (ctx.mem) Number(path, position, index);
+    }
+
+    Signature str_slice_sig = "str-slice($string, $start-at, $end-at:-1)";
+    BUILT_IN(str_slice)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      Number* n = ARG("$start-at", Number);
+      Number* m = ARG("$end-at", Number);
+
+      string str = s->value();
+      char quotemark = s->quote_mark();
+      str = unquote(str);
+
+      // normalize into 0-based indices
+      size_t start = UTF_8::code_point_offset_to_byte_offset(str, UTF_8::normalize_index(n->value(), UTF_8::code_point_count(str)));
+      size_t end = UTF_8::code_point_offset_to_byte_offset(str, UTF_8::normalize_index(m->value(), UTF_8::code_point_count(str)));
+
+      string newstr;
+      if(start - end == 0) {
+        newstr = str.substr(start, end - start);
+      } else {
+        newstr = str.substr(start, end - start + UTF_8::length_of_code_point_at(str, end));
+      }
+      if(quotemark) {
+        newstr = quote(newstr, quotemark);
+      }
+
+      return new (ctx.mem) String_Constant(path, position, newstr);
+
+    }
+
+    Signature to_upper_case_sig = "to-upper-case($string)";
+    BUILT_IN(to_upper_case)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      string str = s->value();
+
+      for (size_t i = 0, L = str.length(); i < L; ++i) {
+        str[i] = std::toupper(str[i]);
+      }
+
+      return new (ctx.mem) String_Constant(path, position, str);
+    }
+
+    Signature to_lower_case_sig = "to-lower-case($string)";
+    BUILT_IN(to_lower_case)
+    {
+      String_Constant* s = ARG("$string", String_Constant);
+      string str = s->value();
+
+      for (size_t i = 0, L = str.length(); i < L; ++i) {
+        str[i] = std::tolower(str[i]);
+      }
+
+      return new (ctx.mem) String_Constant(path, position, str);
+    }
+
     ///////////////////
     // NUMBER FUNCTIONS
     ///////////////////
@@ -751,21 +889,21 @@ namespace Sass {
       List* arglist = ARG("$x2", List);
       Number* least = x1;
       for (size_t i = 0, L = arglist->length(); i < L; ++i) {
-        Number* xi = dynamic_cast<Number*>((*arglist)[i]);
+        Number* xi = dynamic_cast<Number*>(arglist->value_at_index(i));
         if (!xi) error("`" + string(sig) + "` only takes numeric arguments", path, position);
         if (lt(xi, least, ctx)) least = xi;
       }
       return least;
     }
 
-    Signature max_sig = "max($x2, $x2...)";
+    Signature max_sig = "max($x1, $x2...)";
     BUILT_IN(max)
     {
       Number* x1 = ARG("$x1", Number);
       List* arglist = ARG("$x2", List);
       Number* greatest = x1;
       for (size_t i = 0, L = arglist->length(); i < L; ++i) {
-        Number* xi = dynamic_cast<Number*>((*arglist)[i]);
+        Number* xi = dynamic_cast<Number*>(arglist->value_at_index(i));
         if (!xi) error("`" + string(sig) + "` only takes numeric arguments", path, position);
         if (lt(greatest, xi, ctx)) greatest = xi;
       }
@@ -790,13 +928,16 @@ namespace Sass {
     {
       List* l = dynamic_cast<List*>(env["$list"]);
       Number* n = ARG("$n", Number);
+      if (n->value() == 0) error("argument `$n` of `" + string(sig) + "` must be non-zero", path, position);
+      // if the argument isn't a list, then wrap it in a singleton list
       if (!l) {
         l = new (ctx.mem) List(path, position, 1);
         *l << ARG("$list", Expression);
       }
       if (l->empty()) error("argument `$list` of `" + string(sig) + "` must not be empty", path, position);
-      if (n->value() < 1) error("argument `$n` of `" + string(sig) + "` must be greater than or equal to 1", path, position);
-      return (*l)[std::floor(n->value() - 1)];
+      size_t index = std::floor(n->value() < 0 ? l->length() + n->value() : n->value() - 1);
+      if (index < 0 || index > l->length() - 1) error("index out of bounds for `" + string(sig) + "`", path, position);
+      return l->value_at_index(index);
     }
 
     Signature index_sig = "index($list, $value)";
@@ -809,7 +950,7 @@ namespace Sass {
         *l << ARG("$list", Expression);
       }
       for (size_t i = 0, L = l->length(); i < L; ++i) {
-        if (eq((*l)[i], v, ctx)) return new (ctx.mem) Number(path, position, i+1);
+        if (eq(l->value_at_index(i), v, ctx)) return new (ctx.mem) Number(path, position, i+1);
       }
       return new (ctx.mem) Boolean(path, position, false);
     }
@@ -851,7 +992,7 @@ namespace Sass {
         l = new (ctx.mem) List(path, position, 1);
         *l << ARG("$list", Expression);
       }
-      List* result = new (ctx.mem) List(path, position, l->length() + 1);
+      List* result = new (ctx.mem) List(path, position, l->length() + 1, l->separator());
       string sep_str(unquote(sep->value()));
       if (sep_str == "space") result->separator(List::SPACE);
       else if (sep_str == "comma") result->separator(List::COMMA);
@@ -867,11 +1008,15 @@ namespace Sass {
       List* arglist = new (ctx.mem) List(*ARG("$lists", List));
       size_t shortest = 0;
       for (size_t i = 0, L = arglist->length(); i < L; ++i) {
-        List* ith = dynamic_cast<List*>((*arglist)[i]);
+        List* ith = dynamic_cast<List*>(arglist->value_at_index(i));
         if (!ith) {
           ith = new (ctx.mem) List(path, position, 1);
-          *ith << (*arglist)[i];
-          (*arglist)[i] = ith;
+          *ith << arglist->value_at_index(i);
+          if (arglist->is_arglist()) {
+            ((Argument*)(*arglist)[i])->value(ith);
+          } else {
+            (*arglist)[i] = ith;
+          }
         }
         shortest = (i ? std::min(shortest, ith->length()) : ith->length());
       }
@@ -880,7 +1025,7 @@ namespace Sass {
       for (size_t i = 0; i < shortest; ++i) {
         List* zipper = new (ctx.mem) List(path, position, L);
         for (size_t j = 0; j < L; ++j) {
-          *zipper << (*static_cast<List*>((*arglist)[j]))[i];
+          *zipper << (*static_cast<List*>(arglist->value_at_index(j)))[i];
         }
         *zippers << zipper;
       }
@@ -891,20 +1036,22 @@ namespace Sass {
     BUILT_IN(compact)
     {
       List* arglist = ARG("$values", List);
+      List::Separator sep = List::COMMA;
       if (arglist->length() == 1) {
-        Expression* the_arg = (*arglist)[0];
+        Expression* the_arg = arglist->value_at_index(0);
         arglist = dynamic_cast<List*>(the_arg);
         if (!arglist) {
           List* result = new (ctx.mem) List(path, position, 1, List::COMMA);
           *result << the_arg;
           return result;
         }
+        sep = arglist->separator();
       }
-      List* result = new (ctx.mem) List(path, position, 0, List::COMMA);
+      List* result = new (ctx.mem) List(path, position, 0, sep);
       for (size_t i = 0, L = arglist->length(); i < L; ++i) {
-        Boolean* ith = dynamic_cast<Boolean*>((*arglist)[i]);
+        Boolean* ith = dynamic_cast<Boolean*>(arglist->value_at_index(i));
         if (ith && ith->value() == false) continue;
-        *result << (*arglist)[i];
+        *result << arglist->value_at_index(i);
       }
       return result;
     }
@@ -975,4 +1122,4 @@ namespace Sass {
     }
 
   }
-}
\ No newline at end of file
+}
