@@ -5,6 +5,7 @@
 #include "inspect.hpp"
 #include "to_string.hpp"
 #include "constants.hpp"
+#include "util.hpp"
 
 #ifndef SASS_PRELEXER
 #include "prelexer.hpp"
@@ -43,8 +44,8 @@ namespace Sass {
     Selector_Lookahead lookahead_result;
     while (position < end) {
       if (lex< block_comment >()) {
-        String_Constant* contents = new (ctx.mem) String_Constant(path, source_position, lexed);
-        Comment*     comment  = new (ctx.mem) Comment(path, source_position, contents);
+        String*  contents = parse_interpolated_chunk(lexed);
+        Comment* comment  = new (ctx.mem) Comment(path, source_position, contents);
         (*root) << comment;
       }
       else if (peek< import >()) {
@@ -122,9 +123,13 @@ namespace Sass {
     do {
       if (lex< string_constant >()) {
         string import_path(lexed);
-        size_t dot = import_path.find_last_of('.');
-        if (dot != string::npos && import_path.substr(dot, 4) == ".css") {
-          String_Constant* loc = new (ctx.mem) String_Constant(path, source_position, import_path);
+        string extension;
+        if (import_path.length() > 6) { // 2 quote marks + the 4 chars in .css
+          // a string constant is guaranteed to end with a quote mark, so make sure to skip it when indexing from the end
+          extension = import_path.substr(import_path.length() - 5, 4);
+        }
+        if (extension == ".css") {
+          String_Constant* loc = new (ctx.mem) String_Constant(path, source_position, import_path, true);
           Argument* loc_arg = new (ctx.mem) Argument(path, source_position, loc);
           Arguments* loc_args = new (ctx.mem) Arguments(path, source_position);
           (*loc_args) << loc_arg;
@@ -133,7 +138,6 @@ namespace Sass {
         }
         else {
           string current_dir = File::dir_name(path);
-          // string resolved(ctx.add_file(File::join_paths(current_dir, unquote(import_path))));
           string resolved(ctx.add_file(current_dir, unquote(import_path)));
           if (resolved.empty()) error("file to import not found or unreadable: " + import_path);
           imp->files().push_back(resolved);
@@ -158,7 +162,7 @@ namespace Sass {
     else if (lex< function >()) which_type = Definition::FUNCTION;
     string which_str(lexed);
     if (!lex< identifier >()) error("invalid name in " + which_str + " definition");
-    string name(lexed);
+    string name(Util::normalize_underscores(lexed));
     Position source_position_of_def = source_position;
     Parameters* params = parse_parameters();
     if (!peek< exactly<'{'> >()) error("body for " + which_str + " " + name + " must begin with a '{'");
@@ -188,7 +192,7 @@ namespace Sass {
   Parameter* Parser::parse_parameter()
   {
     lex< variable >();
-    string name(lexed);
+    string name(Util::normalize_underscores(lexed));
     Position pos = source_position;
     Expression* val = 0;
     bool is_rest = false;
@@ -208,7 +212,7 @@ namespace Sass {
     lex< include >() /* || lex< exactly<'+'> >() */;
     if (!lex< identifier >()) error("invalid name in @include directive");
     Position source_position_of_call = source_position;
-    string name(lexed);
+    string name(Util::normalize_underscores(lexed));
     Arguments* args = parse_arguments();
     Block* content = 0;
     if (peek< exactly<'{'> >()) {
@@ -240,7 +244,7 @@ namespace Sass {
     Argument* arg;
     if (peek< sequence < variable, spaces_and_comments, exactly<':'> > >()) {
       lex< variable >();
-      string name(lexed);
+      string name(Util::normalize_underscores(lexed));
       Position p = source_position;
       lex< exactly<':'> >();
       Expression* val = parse_space_list();
@@ -262,7 +266,7 @@ namespace Sass {
   Assignment* Parser::parse_assignment()
   {
     lex< variable >();
-    string name(lexed);
+    string name(Util::normalize_underscores(lexed));
     Position var_source_position = source_position;
     if (!lex< exactly<':'> >()) error("expected ':' after " + name + " in assignment statement");
     Expression* val = parse_list();
@@ -341,6 +345,11 @@ namespace Sass {
     To_String to_string;
     Selector_List* group = new (ctx.mem) Selector_List(path, source_position);
     do {
+      if (peek< exactly<'{'> >() ||
+          peek< exactly<'}'> >() ||
+          peek< exactly<')'> >() ||
+          peek< exactly<';'> >())
+        break; // in case there are superfluous commas at the end
       Complex_Selector* comb = parse_selector_combination();
       if (!comb->has_reference()) {
         Position sel_source_position = source_position;
@@ -358,7 +367,7 @@ namespace Sass {
       }
       (*group) << comb;
     }
-    while (lex< exactly<','> >());
+    while (lex< one_plus< sequence< spaces_and_comments, exactly<','> > > >());
     return group;
   }
 
@@ -403,14 +412,26 @@ namespace Sass {
   Compound_Selector* Parser::parse_simple_selector_sequence()
   {
     Compound_Selector* seq = new (ctx.mem) Compound_Selector(path, source_position);
-    // check for backref or type selector, which are only allowed at the front
+    bool sawsomething = false;
     if (lex< exactly<'&'> >()) {
+      // if you see a &
       (*seq) << new (ctx.mem) Selector_Reference(path, source_position);
+      sawsomething = true;
+      // if you see a space after a &, then you're done
+      if(lex< spaces >()) {
+        return seq;
+      }
     }
-    else if (lex< sequence< negate< functional >, alternatives< type_selector, universal, string_constant, dimension, percentage, number > > >()) {
+    if (sawsomething && lex< sequence< negate< functional >, alternatives< hyphens_and_identifier, universal, string_constant, dimension, percentage, number > > >()) {
+      // saw an ampersand, then allow type selectors with arbitrary number of hyphens at the beginning
       (*seq) << new (ctx.mem) Type_Selector(path, source_position, lexed);
+    } else if (lex< sequence< negate< functional >, alternatives< type_selector, universal, string_constant, dimension, percentage, number > > >()) {
+      // if you see a type selector
+      (*seq) << new (ctx.mem) Type_Selector(path, source_position, lexed);
+      sawsomething = true;
     }
-    else {
+    if (!sawsomething) {
+      // don't blindly do this if you saw a & or selector
       (*seq) << parse_simple_selector();
     }
 
@@ -455,22 +476,24 @@ namespace Sass {
     return 0;
   }
 
-  Negated_Selector* Parser::parse_negated_selector()
+  Wrapped_Selector* Parser::parse_negated_selector()
   {
     lex< pseudo_not >();
+    string name(lexed);
     Position nsource_position = source_position;
     Selector* negated = parse_selector_group();
     if (!lex< exactly<')'> >()) {
       error("negated selector is missing ')'");
     }
-    return new (ctx.mem) Negated_Selector(path, nsource_position, negated);
+    return new (ctx.mem) Wrapped_Selector(path, nsource_position, name, negated);
   }
 
-  Pseudo_Selector* Parser::parse_pseudo_selector() {
+  Simple_Selector* Parser::parse_pseudo_selector() {
     if (lex< sequence< pseudo_prefix, functional > >() || lex< functional >()) {
       string name(lexed);
       String* expr = 0;
       Position p = source_position;
+      Selector* wrapped = 0;
       if (lex< alternatives< even, odd > >()) {
         expr = new (ctx.mem) String_Constant(path, p, lexed);
       }
@@ -487,15 +510,21 @@ namespace Sass {
         *schema << var_coef << op << constant;
         expr = schema;
       }
-      else if (lex< sequence< optional<sign>,
-                              optional<digits>,
-                              exactly<'n'> > >()) {
+      else if (peek< sequence< optional<sign>,
+                               optional<digits>,
+                               exactly<'n'>,
+                               spaces_and_comments,
+                               exactly<')'> > >()) {
+        lex< sequence< optional<sign>,
+                       optional<digits>,
+                       exactly<'n'> > >();
         expr = new (ctx.mem) String_Constant(path, p, lexed);
       }
       else if (lex< sequence< optional<sign>, digits > >()) {
         expr = new (ctx.mem) String_Constant(path, p, lexed);
       }
-      else if (lex< identifier >()) {
+      else if (peek< sequence< identifier, spaces_and_comments, exactly<')'> > >()) {
+        lex< identifier >();
         expr = new (ctx.mem) String_Constant(path, p, lexed);
       }
       else if (lex< string_constant >()) {
@@ -505,9 +534,12 @@ namespace Sass {
         expr = new (ctx.mem) String_Constant(path, p, "");
       }
       else {
-        error("invalid argument to " + name + "...)");
+        wrapped = parse_selector_group();
       }
       if (!lex< exactly<')'> >()) error("unterminated argument to " + name + "...)");
+      if (wrapped) {
+        return new (ctx.mem) Wrapped_Selector(path, p, name, wrapped);
+      }
       return new (ctx.mem) Pseudo_Selector(path, p, name, expr);
     }
     else if (lex < sequence< pseudo_prefix, identifier > >()) {
@@ -524,16 +556,26 @@ namespace Sass {
   {
     lex< exactly<'['> >();
     Position p = source_position;
-    if (!lex< type_selector >()) error("invalid attribute name in attribute selector");
+    if (!lex< attribute_name >()) error("invalid attribute name in attribute selector");
     string name(lexed);
-    if (lex< exactly<']'> >()) return new (ctx.mem) Attribute_Selector(path, p, name, "", "");
+    if (lex< exactly<']'> >()) return new (ctx.mem) Attribute_Selector(path, p, name, "", 0);
     if (!lex< alternatives< exact_match, class_match, dash_match,
                             prefix_match, suffix_match, substring_match > >()) {
       error("invalid operator in attribute selector for " + name);
     }
     string matcher(lexed);
-    if (!lex< string_constant >() && !lex< identifier >()) error("expected a string constant or identifier in attribute selector for " + name);
-    string value(lexed);
+
+    String* value = 0;
+    if (lex< identifier >()) {
+      value = new (ctx.mem) String_Constant(path, p, lexed, true);
+    }
+    else if (lex< string_constant >()) {
+      value = parse_interpolated_chunk(lexed);
+    }
+    else {
+      error("expected a string constant or identifier in attribute selector for " + name);
+    }
+
     if (!lex< exactly<']'> >()) error("unterminated attribute selector for " + name);
     return new (ctx.mem) Attribute_Selector(path, p, name, matcher, value);
   }
@@ -551,15 +593,15 @@ namespace Sass {
         }
         semicolon = false;
         while (lex< block_comment >()) {
-          String_Constant* contents = new (ctx.mem) String_Constant(path, source_position, lexed);
-          Comment*         comment  = new (ctx.mem) Comment(path, source_position, contents);
+          String*  contents = parse_interpolated_chunk(lexed);
+          Comment* comment  = new (ctx.mem) Comment(path, source_position, contents);
           (*block) << comment;
         }
         if (lex< exactly<'}'> >()) break;
       }
       if (lex< block_comment >()) {
-        String_Constant* contents = new (ctx.mem) String_Constant(path, source_position, lexed);
-        Comment*         comment  = new (ctx.mem) Comment(path, source_position, contents);
+        String*  contents = parse_interpolated_chunk(lexed);
+        Comment* comment  = new (ctx.mem) Comment(path, source_position, contents);
         (*block) << comment;
       }
       else if (peek< import >(position)) {
@@ -676,8 +718,8 @@ namespace Sass {
       }
       else lex< exactly<';'> >();
       while (lex< block_comment >()) {
-        String_Constant* contents = new (ctx.mem) String_Constant(path, source_position, lexed);
-        Comment*         comment  = new (ctx.mem) Comment(path, source_position, contents);
+        String*  contents = parse_interpolated_chunk(lexed);
+        Comment* comment  = new (ctx.mem) Comment(path, source_position, contents);
         (*block) << comment;
       }
     }
@@ -725,6 +767,15 @@ namespace Sass {
 
     while (lex< exactly<','> >())
     {
+      if (//peek< exactly<'!'> >(position) ||
+          peek< exactly<';'> >(position) ||
+          peek< exactly<'}'> >(position) ||
+          peek< exactly<'{'> >(position) ||
+          peek< exactly<')'> >(position) ||
+          //peek< exactly<':'> >(position) ||
+          peek< exactly<ellipsis> >(position)) {
+        break;
+      }
       Expression* list = parse_space_list();
       (*comma_list) << list;
     }
@@ -823,7 +874,8 @@ namespace Sass {
     Expression* term1 = parse_term();
     // if it's a singleton, return it directly; don't wrap it
     if (!(peek< exactly<'+'> >(position) ||
-          peek< sequence< negate< number >, exactly<'-'> > >(position)))
+          peek< sequence< negate< number >, exactly<'-'> > >(position)) || 
+          peek< identifier >(position))
     { return term1; }
 
     vector<Expression*> operands;
@@ -879,20 +931,25 @@ namespace Sass {
     }
     else if (peek< ie_keyword_arg >()) {
       String_Schema* kwd_arg = new (ctx.mem) String_Schema(path, source_position, 3);
-      if (lex< variable >()) *kwd_arg << new (ctx.mem) Variable(path, source_position, lexed);
+      if (lex< variable >()) *kwd_arg << new (ctx.mem) Variable(path, source_position, Util::normalize_underscores(lexed));
       else {
         lex< alternatives< identifier_schema, identifier > >();
         *kwd_arg << new (ctx.mem) String_Constant(path, source_position, lexed);
       }
       lex< exactly<'='> >();
       *kwd_arg << new (ctx.mem) String_Constant(path, source_position, lexed);
-      if (lex< variable >()) *kwd_arg << new (ctx.mem) Variable(path, source_position, lexed);
+      if (lex< variable >()) *kwd_arg << new (ctx.mem) Variable(path, source_position, Util::normalize_underscores(lexed));
       else {
-        lex< alternatives< identifier_schema, identifier, number > >();
+        lex< alternatives< identifier_schema, identifier, number, hex > >();
         *kwd_arg << new (ctx.mem) String_Constant(path, source_position, lexed);
       }
       return kwd_arg;
     }
+    else if (peek< exactly< calc_kwd > >() ||
+             peek< exactly< moz_calc_kwd > >() ||
+             peek< exactly< webkit_calc_kwd > >()) {
+      return parse_calc_function();
+    }
     else if (peek< functional_schema >()) {
       return parse_function_call_schema();
     }
@@ -916,55 +973,37 @@ namespace Sass {
   Expression* Parser::parse_value()
   {
     if (lex< uri_prefix >()) {
-      // TODO: really need to clean up this chunk
       Arguments* args = new (ctx.mem) Arguments(path, source_position);
       Function_Call* result = new (ctx.mem) Function_Call(path, source_position, "url", args);
-      // gah, gonna have to use exception handling to do backtracking ...
       const char* here = position;
+      Position here_p = source_position;
+      // Try to parse a SassScript expression. If it succeeds and we can munch
+      // a matching rparen, then that's our url. If we can't munch a matching
+      // rparen, or if the attempt to parse an expression fails, then try to
+      // munch a regular CSS url.
       try {
-        try {
-          if (peek< string_constant >()) {
-            // cerr << "parsing a url string" << endl;
-            String* str = parse_string();
-            (*args) << new (ctx.mem) Argument(path, source_position, str);
-            // cerr << "done" << endl;
-          }
-          else if (peek< sequence< url_schema, spaces_and_comments, exactly<')'> > >()) {
-            // cerr << "url schema" << endl;
-            lex< url_schema >();
-            String_Schema* the_url = Parser::from_token(lexed, ctx, path, source_position).parse_url_schema();
-            (*args) << new (ctx.mem) Argument(path, source_position, the_url);
-            // cerr << "done" << endl;
-          }
-          else if (peek< sequence< url_value, spaces_and_comments, exactly<')'> > >()) {
-            // cerr << "url value" << endl;
-            lex< url_value >();
-            String_Constant* the_url = new (ctx.mem) String_Constant(path, source_position, lexed);
-            (*args) << new (ctx.mem) Argument(path, source_position, the_url);
-            // cerr << "done" << endl;
-          }
-          else {
-            // cerr << "stuff" << endl;
-            const char* value = position;
-            const char* rparen = find_first< exactly<')'> >(position);
-            if (!rparen) error("URI is missing ')'");
-            Token content_tok(Token(value, rparen));
-            String_Constant* content_node = new (ctx.mem) String_Constant(path, source_position, content_tok);
-            (*args) << new (ctx.mem) Argument(path, source_position, content_node);
-            position = rparen;
-            // cerr << "done" << endl;
-          }
-        }
-        catch (Error& e) {
-          // cerr << "expression" << endl;
-          position = here;
-          Expression* expr = parse_list();
-          (*args) << new (ctx.mem) Argument(path, source_position, expr);
-          // cerr << "done" << endl;
-        }
+        // special case -- if there's a comment, treat it as part of a URL
+        lex<spaces>();
+        if (peek<line_comment_prefix>() || peek<block_comment_prefix>()) error("comment in URL"); // doesn't really matter what we throw
+        Expression* expr = parse_list();
+        if (!lex< exactly<')'> >()) error("dangling expression in URL"); // doesn't really matter what we throw
+        Argument* arg = new (ctx.mem) Argument(path, expr->position(), expr);
+        *args << arg;
+        return result;
       }
-      catch (Error& e) {
-        error("unable to parse URL");
+      catch (Error& err) {
+        // back up so we can try again
+        position = here;
+        source_position = here_p;
+      }
+      lex< spaces >();
+      if (lex< url >()) {
+        String* the_url = parse_interpolated_chunk(lexed);
+        Argument* arg = new (ctx.mem) Argument(path, the_url->position(), the_url);
+        *args << arg;
+      }
+      else {
+        error("malformed URL");
       }
       if (!lex< exactly<')'> >()) error("URI is missing ')'");
       return result;
@@ -1007,7 +1046,7 @@ namespace Sass {
     { return parse_string(); }
 
     if (lex< variable >())
-    { return new (ctx.mem) Variable(path, source_position, lexed); }
+    { return new (ctx.mem) Variable(path, source_position, Util::normalize_underscores(lexed)); }
 
     error("error reading values after " + lexed.to_string());
 
@@ -1015,28 +1054,26 @@ namespace Sass {
     return 0;
   }
 
-  String* Parser::parse_string()
+  String* Parser::parse_interpolated_chunk(Token chunk)
   {
-    lex< string_constant >();
-    Token str(lexed);
-    const char* i = str.begin;
+    const char* i = chunk.begin;
     // see if there any interpolants
-    const char* p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(str.begin, str.end);
+    const char* p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(chunk.begin, chunk.end);
     if (!p) {
-      String_Constant* str_node = new (ctx.mem) String_Constant(path, source_position, str);
+      String_Constant* str_node = new (ctx.mem) String_Constant(path, source_position, chunk);
       str_node->is_delayed(true);
       return str_node;
     }
 
     String_Schema* schema = new (ctx.mem) String_Schema(path, source_position);
-    schema->quote_mark(*str.begin);
-    while (i < str.end) {
-      p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(i, str.end);
+    schema->quote_mark(*chunk.begin);
+    while (i < chunk.end) {
+      p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(i, chunk.end);
       if (p) {
         if (i < p) {
           (*schema) << new (ctx.mem) String_Constant(path, source_position, Token(i, p)); // accumulate the preceding segment if it's nonempty
         }
-        const char* j = find_first_in_interval< exactly<rbrace> >(p, str.end); // find the closing brace
+        const char* j = find_first_in_interval< exactly<rbrace> >(p, chunk.end); // find the closing brace
         if (j) {
           // parse the interpolant and accumulate it
           Expression* interp_node = Parser::from_token(Token(p+2, j), ctx, path, source_position).parse_list();
@@ -1046,17 +1083,60 @@ namespace Sass {
         }
         else {
           // throw an error if the interpolant is unterminated
-          error("unterminated interpolant inside string constant " + str.to_string());
+          error("unterminated interpolant inside string constant " + chunk.to_string());
         }
       }
       else { // no interpolants left; add the last segment if nonempty
-        if (i < str.end) (*schema) << new (ctx.mem) String_Constant(path, source_position, Token(i, str.end));
+        if (i < chunk.end) (*schema) << new (ctx.mem) String_Constant(path, source_position, Token(i, chunk.end));
         break;
       }
     }
     return schema;
   }
 
+  String* Parser::parse_string()
+  {
+    lex< string_constant >();
+    Token str(lexed);
+    return parse_interpolated_chunk(str);
+    // const char* i = str.begin;
+    // // see if there any interpolants
+    // const char* p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(str.begin, str.end);
+    // if (!p) {
+    //   String_Constant* str_node = new (ctx.mem) String_Constant(path, source_position, str);
+    //   str_node->is_delayed(true);
+    //   return str_node;
+    // }
+
+    // String_Schema* schema = new (ctx.mem) String_Schema(path, source_position);
+    // schema->quote_mark(*str.begin);
+    // while (i < str.end) {
+    //   p = find_first_in_interval< sequence< negate< exactly<'\\'> >, exactly<hash_lbrace> > >(i, str.end);
+    //   if (p) {
+    //     if (i < p) {
+    //       (*schema) << new (ctx.mem) String_Constant(path, source_position, Token(i, p)); // accumulate the preceding segment if it's nonempty
+    //     }
+    //     const char* j = find_first_in_interval< exactly<rbrace> >(p, str.end); // find the closing brace
+    //     if (j) {
+    //       // parse the interpolant and accumulate it
+    //       Expression* interp_node = Parser::from_token(Token(p+2, j), ctx, path, source_position).parse_list();
+    //       interp_node->is_interpolant(true);
+    //       (*schema) << interp_node;
+    //       i = j+1;
+    //     }
+    //     else {
+    //       // throw an error if the interpolant is unterminated
+    //       error("unterminated interpolant inside string constant " + str.to_string());
+    //     }
+    //   }
+    //   else { // no interpolants left; add the last segment if nonempty
+    //     if (i < str.end) (*schema) << new (ctx.mem) String_Constant(path, source_position, Token(i, str.end));
+    //     break;
+    //   }
+    // }
+    // return schema;
+  }
+
   String* Parser::parse_ie_stuff()
   {
     lex< ie_stuff >();
@@ -1131,7 +1211,7 @@ namespace Sass {
         if (!num_items) schema->quote_mark(*lexed.begin);
       }
       else if (lex< variable >()) {
-        (*schema) << new (ctx.mem) Variable(path, source_position, lexed);
+        (*schema) << new (ctx.mem) Variable(path, source_position, Util::normalize_underscores(lexed));
       }
       else {
         error("error parsing interpolated value");
@@ -1209,10 +1289,28 @@ namespace Sass {
     return schema;
   }
 
-  Function_Call* Parser::parse_function_call()
+  Function_Call* Parser::parse_calc_function()
   {
     lex< identifier >();
     string name(lexed);
+    Position call_pos = source_position;
+    lex< exactly<'('> >();
+    Position arg_pos = source_position;
+    const char* arg_beg = position;
+    parse_list();
+    const char* arg_end = position;
+    lex< exactly<')'> >();
+
+    Argument* arg = new (ctx.mem) Argument(path, arg_pos, parse_interpolated_chunk(Token(arg_beg, arg_end)));
+    Arguments* args = new (ctx.mem) Arguments(path, arg_pos);
+    *args << arg;
+    return new (ctx.mem) Function_Call(path, call_pos, name, args);
+  }
+
+  Function_Call* Parser::parse_function_call()
+  {
+    lex< identifier >();
+    string name(Util::normalize_underscores(lexed));
     Position source_position_of_call = source_position;
 
     Function_Call* the_call = new (ctx.mem) Function_Call(path, source_position_of_call, name, parse_arguments());
@@ -1372,6 +1470,7 @@ namespace Sass {
     string kwd(lexed);
     Position at_source_position = source_position;
     Selector* sel = 0;
+    Expression* val = 0;
     Selector_Lookahead lookahead = lookahead_for_extension_target(position);
     if (lookahead.found) {
       if (lookahead.has_interpolants) {
@@ -1381,9 +1480,14 @@ namespace Sass {
         sel = parse_selector_group();
       }
     }
+    else if (!(peek<exactly<'{'> >() || peek<exactly<'}'> >() || peek<exactly<';'> >())) {
+      val = parse_list();
+    }
     Block* body = 0;
     if (peek< exactly<'{'> >()) body = parse_block();
-    return new (ctx.mem) At_Rule(path, at_source_position, kwd, sel, body);
+    At_Rule* rule = new (ctx.mem) At_Rule(path, at_source_position, kwd, sel, body);
+    if (!sel) rule->value(val);
+    return rule;
   }
 
   Warning* Parser::parse_warning()
@@ -1396,9 +1500,11 @@ namespace Sass {
   {
     const char* p = start ? start : position;
     const char* q;
+    bool saw_stuff = false;
     bool saw_interpolant = false;
 
     while ((q = peek< identifier >(p))                             ||
+           (q = peek< hyphens_and_identifier >(p))                 ||
            (q = peek< type_selector >(p))                          ||
            (q = peek< id_name >(p))                                ||
            (q = peek< class_name >(p))                             ||
@@ -1435,12 +1541,13 @@ namespace Sass {
            (q = peek< sequence< exactly<'-'>, interpolant > >(p))  ||
            (q = peek< sequence< pseudo_prefix, interpolant > >(p)) ||
            (q = peek< interpolant >(p))) {
+      saw_stuff = true;
       p = q;
       if (*(p - 1) == '}') saw_interpolant = true;
     }
 
     Selector_Lookahead result;
-    result.found            = peek< exactly<'{'> >(p) ? p : 0;
+    result.found            = saw_stuff && peek< exactly<'{'> >(p) ? p : 0;
     result.has_interpolants = saw_interpolant;
 
     return result;
